@@ -1,7 +1,7 @@
 """PyPI and direct package downloading."""
 
-import sys
 import subprocess
+import sys
 import os
 import re
 import io
@@ -113,15 +113,13 @@ def egg_info_for_url(url):
 def distros_for_url(url, metadata=None):
     """Yield egg or source distribution objects that might be found at a URL"""
     base, fragment = egg_info_for_url(url)
-    for dist in distros_for_location(url, base, metadata):
-        yield dist
+    yield from distros_for_location(url, base, metadata)
     if fragment:
         match = EGG_FRAGMENT.match(fragment)
         if match:
-            for dist in interpret_distro_name(
+            yield from interpret_distro_name(
                 url, match.group(1), metadata, precedence=CHECKOUT_DIST
-            ):
-                yield dist
+            )
 
 
 def distros_for_location(location, basename, metadata=None):
@@ -196,7 +194,7 @@ def interpret_distro_name(
         '-'.join(parts[p:]),
         py_version=py_version,
         precedence=precedence,
-        platform=platform
+        platform=platform,
     )
 
 
@@ -306,7 +304,7 @@ class PackageIndex(Environment):
         ca_bundle=None,
         verify_ssl=True,
         *args,
-        **kw
+        **kw,
     ):
         super().__init__(*args, **kw)
         self.index_url = index_url + "/"[: not index_url.endswith('/')]
@@ -322,7 +320,7 @@ class PackageIndex(Environment):
         try:
             parse_version(dist.version)
         except Exception:
-            return
+            return None
         return super().add(dist)
 
     # FIXME: 'PackageIndex.process_url' is too complex (14)
@@ -409,6 +407,7 @@ class PackageIndex(Environment):
             raise DistutilsError(msg % url)
         else:
             self.warn(msg, url)
+            return False
 
     def scan_egg_links(self, search_path):
         dirs = filter(os.path.isdir, search_path)
@@ -517,7 +516,7 @@ class PackageIndex(Environment):
             if dist in requirement:
                 return dist
             self.debug("%s does not match %s", requirement, dist)
-        return super(PackageIndex, self).obtain(requirement, installer)
+        return super().obtain(requirement, installer)
 
     def check_hash(self, checker, filename, tfp):
         """
@@ -587,7 +586,7 @@ class PackageIndex(Environment):
             scheme = URL_SCHEME(spec)
             if scheme:
                 # It's a url, download it to tmpdir
-                found = self._download_url(spec, tmpdir)
+                found = self._download_url(scheme.group(1), spec, tmpdir)
                 base, fragment = egg_info_for_url(spec)
                 if base.endswith('.py'):
                     found = self.gen_setup(found, fragment, tmpdir)
@@ -635,7 +634,6 @@ class PackageIndex(Environment):
             # Find a matching distribution; may be called more than once
 
             for dist in env[req.key]:
-
                 if dist.precedence == DEVELOP_DIST and not develop_ok:
                     if dist not in skipped:
                         self.warn(
@@ -652,6 +650,8 @@ class PackageIndex(Environment):
                     if os.path.exists(dist.download_location):
                         return dist
 
+            return None
+
         if force_scan:
             self.prescan()
             self.find_packages(requirement)
@@ -675,6 +675,7 @@ class PackageIndex(Environment):
                 (source and "a source distribution of " or ""),
                 requirement,
             )
+            return None
         else:
             self.info("Best match: %s", dist)
             return dist.clone(location=dist.download_location)
@@ -808,13 +809,13 @@ class PackageIndex(Environment):
                     '%s returned a bad status line. The server might be '
                     'down, %s' % (url, v.line)
                 ) from v
-        except (http.client.HTTPException, socket.error) as v:
+        except (http.client.HTTPException, OSError) as v:
             if warning:
                 self.warn(warning, v)
             else:
                 raise DistutilsError("Download error for %s: %s" % (url, v)) from v
 
-    def _download_url(self, url, tmpdir):
+    def _download_url(self, scheme, url, tmpdir):
         # Determine download filename
         #
         name, fragment = egg_info_for_url(url)
@@ -829,60 +830,19 @@ class PackageIndex(Environment):
 
         filename = os.path.join(tmpdir, name)
 
-        return self._download_vcs(url, filename) or self._download_other(url, filename)
-
-
-    @staticmethod
-    def _resolve_vcs(url):
-        """
-        >>> rvcs = PackageIndex._resolve_vcs
-        >>> rvcs('git+http://foo/bar')
-        'git'
-        >>> rvcs('hg+https://foo/bar')
-        'hg'
-        >>> rvcs('git:myhost')
-        'git'
-        >>> rvcs('hg:myhost')
-        >>> rvcs('http://foo/bar')
-        """
-        scheme = urllib.parse.urlsplit(url).scheme
-        pre, sep, post = scheme.partition('+')
-        # svn and git have their own protocol; hg does not
-        allowed = set(['svn', 'git'] + ['hg'] * bool(sep))
-        return next(iter({pre} & allowed), None)
-
-    def _download_vcs(self, url, spec_filename):
-        vcs = self._resolve_vcs(url)
-        if not vcs:
-            return
-        if vcs == 'svn':
-            raise DistutilsError(
-                f"Invalid config, SVN download is not supported: {url}"
-            )
-
-        filename, _, _ = spec_filename.partition('#')
-        url, rev = self._vcs_split_rev_from_url(url)
-
-        self.info(f"Doing {vcs} clone from {url} to {filename}")
-        subprocess.check_call([vcs, 'clone', '--quiet', url, filename])
-
-        co_commands = dict(
-            git=[vcs, '-C', filename, 'checkout', '--quiet', rev],
-            hg=[vcs, '--cwd', filename, 'up', '-C', '-r', rev, '-q'],
-        )
-        if rev is not None:
-            self.info(f"Checking out {rev}")
-            subprocess.check_call(co_commands[vcs])
-
-        return filename
-
-    def _download_other(self, url, filename):
-        scheme = urllib.parse.urlsplit(url).scheme
-        if scheme == 'file':  # pragma: no cover
-            return urllib.request.url2pathname(urllib.parse.urlparse(url).path)
-        # raise error if not allowed
-        self.url_ok(url, True)
-        return self._attempt_download(url, filename)
+        # Download the file
+        #
+        if scheme == 'svn' or scheme.startswith('svn+'):
+            return self._download_svn(url, filename)
+        elif scheme == 'git' or scheme.startswith('git+'):
+            return self._download_git(url, filename)
+        elif scheme.startswith('hg+'):
+            return self._download_hg(url, filename)
+        elif scheme == 'file':
+            return urllib.request.url2pathname(urllib.parse.urlparse(url)[2])
+        else:
+            self.url_ok(url, True)  # raises error if not allowed
+            return self._attempt_download(url, filename)
 
     def scan_url(self, url):
         self.process_url(url, True)
@@ -898,36 +858,52 @@ class PackageIndex(Environment):
         os.unlink(filename)
         raise DistutilsError(f"Unexpected HTML page found at {url}")
 
-    @staticmethod
-    def _vcs_split_rev_from_url(url):
-        """
-        Given a possible VCS URL, return a clean URL and resolved revision if any.
-        >>> vsrfu = PackageIndex._vcs_split_rev_from_url
-        >>> vsrfu('git+https://github.com/pypa/setuptools@v69.0.0#egg-info=setuptools')
-        ('https://github.com/pypa/setuptools', 'v69.0.0')
-        >>> vsrfu('git+https://github.com/pypa/setuptools#egg-info=setuptools')
-        ('https://github.com/pypa/setuptools', None)
-        >>> vsrfu('http://foo/bar')
-        ('http://foo/bar', None)
-        """
-        parts = urllib.parse.urlsplit(url)
+    def _download_svn(self, url, _filename):
+        raise DistutilsError(f"Invalid config, SVN download is not supported: {url}")
 
-        clean_scheme = parts.scheme.split('+', 1)[-1]
+    @staticmethod
+    def _vcs_split_rev_from_url(url, pop_prefix=False):
+        scheme, netloc, path, query, frag = urllib.parse.urlsplit(url)
+
+        scheme = scheme.split('+', 1)[-1]
 
         # Some fragment identification fails
-        no_fragment_path, _, _ = parts.path.partition('#')
+        path = path.split('#', 1)[0]
 
-        pre, sep, post = no_fragment_path.rpartition('@')
-        clean_path, rev = (pre, post) if sep else (post, None)
+        rev = None
+        if '@' in path:
+            path, rev = path.rsplit('@', 1)
 
-        resolved = parts._replace(
-            scheme=clean_scheme,
-            path=clean_path,
-            # discard the fragment
-            fragment='',
-        ).geturl()
+        # Also, discard fragment
+        url = urllib.parse.urlunsplit((scheme, netloc, path, query, ''))
 
-        return resolved, rev
+        return url, rev
+
+    def _download_git(self, url, filename):
+        filename = filename.split('#', 1)[0]
+        url, rev = self._vcs_split_rev_from_url(url, pop_prefix=True)
+
+        self.info("Doing git clone from %s to %s", url, filename)
+        subprocess.check_call(["git", "clone", "--quiet", url, filename])
+
+        if rev is not None:
+            self.info("Checking out %s", rev)
+            subprocess.check_call(["git", "-C", filename, "checkout", "--quiet", rev])
+
+        return filename
+
+    def _download_hg(self, url, filename):
+        filename = filename.split('#', 1)[0]
+        url, rev = self._vcs_split_rev_from_url(url, pop_prefix=True)
+
+        self.info("Doing hg clone from %s to %s", url, filename)
+        subprocess.check_call(["hg", "clone", "--quiet", url, filename])
+
+        if rev is not None:
+            self.info("Updating to %s", rev)
+            subprocess.check_call(["hg", "--cwd", filename, "up", "-C", "-r", rev, "-q"])
+
+        return filename
 
     def debug(self, msg, *args):
         log.debug(msg, *args)
@@ -1051,6 +1027,7 @@ class PyPIConfig(configparser.RawConfigParser):
         for repository, cred in self.creds_by_repository.items():
             if url.startswith(repository):
                 return cred
+        return None
 
 
 def open_with_auth(url, opener=urllib.request.urlopen):
